#!/bin/bash
set -e
umask 0022

PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

if [ -n "$BASH_SOURCE" ]; then
  TOP_DIR="`dirname $BASH_SOURCE`"
else
  TOP_DIR="`pwd`"
fi
TOP_DIR=`readlink -f "$TOP_DIR"`
export TOP_DIR

# load app vars
#. $TOP_DIR/config_app.sh

PKG_DIR=$TOP_DIR
APT_PKG_LIST=$PKG_DIR/apt_packages.txt
APT_PKG_FILE=$PKG_DIR/apt_packages.zip
PIP_PKG_LIST=$PKG_DIR/pip_packages.txt
LUA_PKG_LIST=$PKG_DIR/lua_packages.txt
DEB_PKG_DIR=$PKG_DIR/deb
LUA_PKG_DIR=$PKG_DIR/lua
PIP_PKG_DIR=$PKG_DIR/pip



if [[ -f $APT_PKG_LIST && -s $APT_PKG_LIST ]]; then
    # Add apt archive to apt DB
    if [ -f "$APT_PKG_FILE" ]; then
        echo "Found a new apt archive, adding to local apt DB..."
        sudo apt-offline install $APT_PKG_FILE || :
    fi

    # Install new apt packages
    declare -a pkg_list
    readarray -t pkg_list < $APT_PKG_LIST

    if [ ${#pkg_list[@]} -eq 0 ]; then
        echo "Nothing to install for additional apt packages."
    else
        for f in "${pkg_list[@]}"
        do
            if [ ! -z $f ]; then
                echo "Installing" $f ...
                sudo apt -y install $f || :
            fi
        done
        echo "Done with installing additional apt packages!"
    fi
fi

# Install additional deb packages
if [[ -d $DEB_PKG_DIR ]]; then
    cd $DEB_PKG_DIR
    sudo dpkg -i -R $DEB_PKG_DIR || :
    echo "Done with installing additional deb packages!"
fi

# Install additional pip packages
if [[ -f $PIP_PKG_LIST && -s $PIP_PKG_LIST && -d $PIP_PKG_DIR ]]; then
    cd $PIP_PKG_DIR
    sudo pip install --ignore-installed --no-index --find-links $PIP_PKG_DIR -r $PIP_PKG_LIST || :
    echo "Done with installing additional pip packages!"
fi

# Install additional lua packages
if [[ -f $LUA_PKG_LIST && -s $LUA_PKG_LIST && -d $LUA_PKG_DIR ]]; then
    cd $LUA_PKG_DIR
    pkg_list=(*.rock)
    if [ ${#pkg_list[@]} -eq 0 ]; then
        echo "Nothing to install for additional lua packages."
    else
        for f in "${pkg_list[@]}"
        do
            if [ ! -z $f ]; then
                echo "Installing" $f ...
                sudo luarocks install $f || :
            fi
        done
        echo "Done with installing additional lua packages!"
    fi
fi

exit 0

