#!/bin/bash
set -e

PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

if [ -n "$BASH_SOURCE" ]; then
  TOP_DIR="`dirname $BASH_SOURCE`"
else
  TOP_DIR="`pwd`"
fi
TOP_DIR=`readlink -f "$TOP_DIR"`
export TOP_DIR

# load app vars
. $TOP_DIR/config_app.sh

TMP_DIR=/tmp
UPDATE_DIR=/tmp/update
NETCTRL_DIR=/home/${NETCTRL_USER}/${NAME}
NETCTRL_CURR_DIR=/home/${NETCTRL_USER}/${NAME}-curr
NETCTRL_PREV_DIR=/home/${NETCTRL_USER}/${NAME}-prev
TMP_DB_DIR=/tmp/update/backup_db

# mail constants ONC-657-CONSTANTS
NETCTRL_DATA_DIR=/data
NETCTRL_PREV_DB_DIR=${NETCTRL_DATA_DIR}/previous_db
NETCTRL_SMTP_DIRNAME=smtp
NETCTRL_SMTP_DIR=${NETCTRL_DATA_DIR}/$NETCTRL_SMTP_DIRNAME
NETCTRL_PREV_SMTP_DIR=${NETCTRL_PREV_DB_DIR}/$NETCTRL_SMTP_DIRNAME
NETCTRL_OUTBOX_DIRNAME=outbox
NETCTRL_OUTBOX_DIR=${NETCTRL_DATA_DIR}/$NETCTRL_OUTBOX_DIRNAME

function cleanup_mail {
    rm -fr $NETCTRL_SMTP_DIR
    rm -fr $NETCTRL_OUTBOX_DIR
    rm -fr $NETCTRL_PREV_SMTP_DIR
}

if [ -z $MODEL ]; then
    MODEL=nc600
fi

HAS_UPDATE=

if [ ! -f $NETCTRL_LOG ]; then
    sudo touch $NETCTRL_LOG
    sudo chown ${NETCTRL_USER}:${NETCTRL_GROUP} $NETCTRL_LOG
    sudo chmod 666 $NETCTRL_LOG
fi

#
##################################################
#
# Functions to get/set version info in system DB.
#
##################################################
#
# table [SYSTEM_CONFIG_TBL]
# cmd_file [sw1_filename | sw2_filename | fd_filename | update_filename]
# cmd_version [current_sw | previous_sw | sw1_version | sw2_version | fd_version | update_version]
#
# table [IP_CONFIG_TBL]
# cmd_local [ip_addr | subnet | gateway]
# cmd_vlan [vlan_ip_addr | vlan_subnet | vlan_gateway | vlan_id]
#
function system_db_set {
    sudo sqlite3 ${NETCTRL_DB_DIR}/system.db "update $table set $element='$value'"
}

function system_db_get {
    local value=`sqlite3 ${NETCTRL_DB_DIR}/system.db "select $element from $table"`
    echo $value
}

#
##################################################
#
# Functions to start/stop Optigo apps.
#
##################################################
#
function sleep_n_stop_apps {
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Stopping all apps..." >>$NETCTRL_LOG 2>&1
#    sleep 1
    sudo service monit stop
    sudo /etc/init.d/optigo_netctrl stop
    set +e
    sudo killall -9 optigo_data_api # make extra sure that nwksvc is dead
    set -e
    sudo service lighttpd stop
}

function stop_apps {
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Stopping all apps..." >>$NETCTRL_LOG 2>&1
    sudo service monit stop
    sudo /etc/init.d/optigo_netctrl stop
    set +e
    sudo killall -9 optigo_data_api # make extra sure that nwksvc is dead
    set -e
    sudo service lighttpd stop
}

function start_apps {
    sudo /etc/init.d/optigo_netctrl start
    sudo service lighttpd start
    sudo service monit start
}

function reset_unit {
    sudo sync
#    sleep 3
    sudo reboot -h now
}


#
##################################################
#
# Functions to handle SW update.
#
##################################################
#
function swup_clean {
    # get fd_version
    element='fd_version'
    fd_version=`get_version`
    if [ -z $fd_version ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] ERROR: Empty default version! Aborting..." >>$NETCTRL_LOG 2>&1
        exit 1
    fi
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: fd_version = $fd_version" >>$NETCTRL_LOG 2>&1
    if [[ -f ${NETCTRL_FW_DIR}/${PRODUCT}-${NAME}-${fd_version}.optg ]]; then
        filename=${PRODUCT}-${NAME}-${fd_version}.optg
    elif [[ -f ${NETCTRL_FW_DIR}/${PRODUCT}-${MODEL}-fw-${fd_version}.optg ]]; then
        filename=${PRODUCT}-${MODEL}-fw-${fd_version}.optg
    fi
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: fd_filename = $filename" >>$NETCTRL_LOG 2>&1

    # clean up files in firmware; rm files that are not the factory default */
    cd ${NETCTRL_FW_DIR}
    rm -f `ls -l | grep -v $filename | awk '{print $9}'`
}

function swup_revert_releases {
    sudo mv -f ${NETCTRL_DIR} ${NETCTRL_CURR_DIR}
    sudo mv -f ${NETCTRL_PREV_DIR} ${NETCTRL_DIR}
    sudo mv -f ${NETCTRL_CURR_DIR} ${NETCTRL_PREV_DIR}
    sudo chown -R ${NETCTRL_USER}:${NETCTRL_GROUP} ${NETCTRL_PREV_DIR}
    sudo chown -R ${NETCTRL_USER}:${NETCTRL_GROUP} ${NETCTRL_DIR}
}

function swup_default {
    sudo rm -fr ${NETCTRL_PREV_DIR}
}

function swup_restore {
    sudo rm -fr ${UPDATE_DIR}
    mkdir -p ${UPDATE_DIR}

    f0=$(echo $filename | sed "s/netctrl/nc600-fw/")
    f1=$(echo $filename | sed "s/nc600-fw/netctrl/")

    if [ -e ${NETCTRL_FW_DIR}/$f0 ]; then
        cp -af ${NETCTRL_FW_DIR}/$f0 ${UPDATE_DIR}/$filename
    elif [ -e ${NETCTRL_FW_DIR}/$f1 ]; then
        cp -af ${NETCTRL_FW_DIR}/$f1 ${UPDATE_DIR}/$filename
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] ERROR: No image file found: $f0 or ${f1}. Aborting!" >>$NETCTRL_LOG 2>&1
        exit 1
    fi
}

function swup_backup {
    if [ -e ${UPDATE_DIR}/$filename ]; then
        cd ${UPDATE_DIR}
        sudo chown ${NETCTRL_USER}:${NETCTRL_GROUP} $filename
        chmod 644 $filename
        cp -af $filename ${NETCTRL_FW_DIR}/
    fi
}

function swup_image {
    cd ${UPDATE_DIR}
    VERSION=`cat ${NETCTRL_DIR}/version.txt`
    if [[ -f ${UPDATE_DIR}/${PRODUCT}-${NAME}-$VERSION.run ]]; then
        NETCTRL_RELEASE_SFX=${PRODUCT}-${NAME}-$VERSION.run
        echo "SFX archive $NETCTRL_RELEASE_SFX"
        NETCTRL_RELEASE_OPTG=${PRODUCT}-${NAME}-$VERSION.optg
    elif [[ -f ${UPDATE_DIR}/${PRODUCT}-${MODEL}-fw-$VERSION.run ]]; then
        NETCTRL_RELEASE_SFX=${PRODUCT}-${MODEL}-fw-$VERSION.run
        echo "SFX archive $NETCTRL_RELEASE_SFX"
        NETCTRL_RELEASE_OPTG=${PRODUCT}-${MODEL}-fw-$VERSION.optg
    fi
    # generate package zip in .optg extension
    rm -fr ${STAGING_DIR}
    mkdir -p ${STAGING_DIR}
    cd ${STAGING_DIR}
    cp -af ${UPDATE_DIR}/$NETCTRL_RELEASE_SFX .
    cp -af ${NETCTRL_DIR}/version.txt .
    zip -q $NETCTRL_RELEASE_OPTG.zip *
    mv -f $NETCTRL_RELEASE_OPTG.zip ${NETCTRL_FW_DIR}/$NETCTRL_RELEASE_OPTG
    cd ${UPDATE_DIR}
    sudo chown ${NETCTRL_USER}:${NETCTRL_GROUP} ${NETCTRL_FW_DIR}/$NETCTRL_RELEASE_OPTG
    chmod 644 ${NETCTRL_FW_DIR}/$NETCTRL_RELEASE_OPTG
    rm -fr ${STAGING_DIR}
}

function swup_extract {
    cd ${UPDATE_DIR}
    if [ ! -e $filename.zip ]; then
        cp -af $filename $filename.zip
        unzip -q -o $filename.zip
    fi
}

function swup_check {
    cd ${UPDATE_DIR}
    if [ -e ${UPDATE_DIR}/version.txt ]; then
        UPDATE_VERSION=`cat ${UPDATE_DIR}/version.txt`
        if [ -e ${NETCTRL_DIR}/version.txt ]; then
            CURRENT_VERSION=`cat ${NETCTRL_DIR}/version.txt`
        else
            # force update on an old version
            HAS_UPDATE=1
            return
        fi
        # compare version
        ret=$($TOP_DIR/version_compare.sh $UPDATE_VERSION $CURRENT_VERSION)
        if [ "${ret}" == "-1" ]; then
            echo "Error, cannot downgrade!"
            HAS_UPDATE=-1
        elif [ "${ret}" == "1" ]; then
            echo "New update is available ..."
            HAS_UPDATE=1
        else
            echo "Same, do nothing."
            HAS_UPDATE=0
        fi
    else
        HAS_UPDATE=0
    fi
}

function swup_upgrade {
    cd ${UPDATE_DIR}
    UPDATE_VERSION=`cat ${UPDATE_DIR}/version.txt`
    echo "${UPDATE_DIR}/${PRODUCT}-${NAME}-$UPDATE_VERSION.run"
    echo "${UPDATE_DIR}/${PRODUCT}-${MODEL}-fw-$UPDATE_VERSION.run"
    if [[ -e ${UPDATE_DIR}/${PRODUCT}-${NAME}-$UPDATE_VERSION.run ]]; then
        NETCTRL_RELEASE_SFX=${PRODUCT}-${NAME}-$UPDATE_VERSION.run
        echo "SFX archive $NETCTRL_RELEASE_SFX"
    elif [[ -e ${UPDATE_DIR}/${PRODUCT}-${MODEL}-fw-$UPDATE_VERSION.run ]]; then
        NETCTRL_RELEASE_SFX=${PRODUCT}-${MODEL}-fw-$UPDATE_VERSION.run
        echo "SFX archive $NETCTRL_RELEASE_SFX"
    fi
    echo "SFX archive $NETCTRL_RELEASE_SFX"
    # Extract archive file into update folder
    if [ -e ${UPDATE_DIR}/$NETCTRL_RELEASE_SFX ]; then
        sudo chown ${NETCTRL_USER}:${NETCTRL_GROUP} ./$NETCTRL_RELEASE_SFX
        chmod 775 ./$NETCTRL_RELEASE_SFX

        # ONC-605: need to modify files before execution
        ./$NETCTRL_RELEASE_SFX --noexec
        cd release/netctrl
        if [ "swup_update.sh" -ot "$TOP_DIR/swup_update.sh" ]; then
            echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Preserving current newer update script" >>$NETCTRL_LOG 2>&1
            cp -p $TOP_DIR/swup_update.sh ./
        fi
        if [ "swup_ons_s8_fw.sh" -ot "$TOP_DIR/swup_ons_s8_fw.sh" ]; then
            echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Preserving current newer S8 update script" >>$NETCTRL_LOG 2>&1
            cp -p $TOP_DIR/swup_ons_s8_fw.sh ./
        fi
        cd ../..
        ./release/netctrl/update_app.sh
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] ERROR: No update file found. Aborting!" >>$NETCTRL_LOG 2>&1
    fi
}

#
##################################################
#
# Functions to handle DBs.
#
##################################################
#
function swup_migrate_db {
#    sudo killall -15 optigo_data_api || :
#    sleep 2
    sudo ${NETCTRL_SCRIPTS_DIR}/migration/migrate.sh
}

function swup_revert_dbs {
    sudo mkdir -p ${TMP_DB_DIR}

    sudo cp -af ${NETCTRL_DB_DIR}/*.db ${TMP_DB_DIR}/
    sudo rm -f ${NETCTRL_DB_DIR}/*.db
    if [ -f $NETCTRL_SMTP_DIR]; then
        sudo cp -afr $NETCTRL_SMTP_DIR $TMP_DB_DIR/
        sudo rm -fr $NETCTRL_SMTP_DIR $NETCTRL_OUTBOX_DIR
    fi

    sudo cp -af ${NETCTRL_PREV_DB_DIR}/*.db ${NETCTRL_DB_DIR}/
    sudo chown -R ${NETCTRL_USER}:${NETCTRL_GROUP} ${NETCTRL_DB_DIR}/*
    sudo rm -f ${NETCTRL_PREV_DB_DIR}/*.db
    if [ -f $NETCTRL_PREV_SMTP_DIR ]; then
	    sudo cp -afr $NETCTRL_PREV_SMTP_DIR $NETCTRL_SMTP_DIR
	    sudo chown -R ${NETCTRL_USER}:${NETCTRL_GROUP} $NETCTRL_SMTP_DIR
	    sudo rm -fr $NETCTRL_PREV_SMTP_DIR
    fi

    sudo cp -af ${TMP_DB_DIR}/*.db ${NETCTRL_PREV_DB_DIR}/
    if [ -f $TMP_DB_DIR/$NETCTRL_SMTP_DIRNAME ]; then
	    sudo cp -afr $TMP_DB_DIR/$NETCTRL_SMTP_DIRNAME $NETCTRL_PREV_SMTP_DIR
    fi
    sudo chown -R ${NETCTRL_USER}:${NETCTRL_GROUP} ${NETCTRL_PREV_DB_DIR}/*
    sudo rm -fr ${TMP_DB_DIR}
}

function swup_backup_dbs {
    sudo mkdir -p ${NETCTRL_PREV_DB_DIR}
    sudo rm -f ${NETCTRL_PREV_DB_DIR}/*.db
    sudo cp -af ${NETCTRL_DB_DIR}/*.db ${NETCTRL_PREV_DB_DIR}/

    # do mail files
    sudo rm -fr $NETCTRL_PREV_SMTP_DIR
    if [ -f $NETCTRL_SMTP_DIR ]; then
        sudo cp -afr ${NETCTRL_SMTP_DIR} ${NETCTRL_PREV_SMTP_DIR}
    fi

    sudo chown -R ${NETCTRL_USER}:${NETCTRL_GROUP} ${NETCTRL_PREV_DB_DIR}/*
}

function swup_reset_default_db {
    cleanup_mail
    sudo ${NETCTRL_SCRIPTS_DIR}/optigo_set_default_bank.sh
    sudo chown -R ${NETCTRL_USER}:${NETCTRL_GROUP} ${NETCTRL_DB_DIR}/*
}

#
##################################################
#
# Functions to handle Management IPs.
#
##################################################
#
# table [IP_CONFIG_TBL]
# cmd_local [ip_addr | subnet | gateway]
# cmd_vlan [vlan_ip_addr | vlan_subnet | vlan_gateway | vlan_id]
ipaddr=
mask=
gateway=
vlan_ipaddr=
vlan_mask=
vlan_gateway=
vlan_id=
function swup_backup_ips {
    # get ip_addr
    element='ip_addr'
    ipaddr=`get_mgmtip`
    if [ -z $ipaddr ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty local IP address!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: ip_addr = $ipaddr" >>$NETCTRL_LOG 2>&1
    fi
    # get mask
    element='subnet'
    mask=`get_mgmtip`
    if [ -z $mask ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty local subnet!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: subnet = $mask" >>$NETCTRL_LOG 2>&1
    fi
    # get gateway
    element='gateway'
    gateway=`get_mgmtip`
    if [ -z $gateway ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty local gateway!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: gateway = $gateway" >>$NETCTRL_LOG 2>&1
    fi
    # get vlan_ip_addr
    element='vlan_ip_addr'
    vlan_ipaddr=`get_mgmtip`
    if [ -z $vlan_ipaddr ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty vlan IP address!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: vlan_ip_addr = $vlan_ipaddr" >>$NETCTRL_LOG 2>&1
    fi
    # get vlan_subnet
    element='vlan_subnet'
    vlan_mask=`get_mgmtip`
    if [ -z $vlan_mask ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty vlan subnet!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: vlan_subnet = $vlan_mask" >>$NETCTRL_LOG 2>&1
    fi
    # get vlan_gateway
    element='vlan_gateway'
    vlan_gateway=`get_mgmtip`
    if [ -z $vlan_gateway ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty vlan gateway!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: vlan_gateway = $vlan_gateway" >>$NETCTRL_LOG 2>&1
    fi
    # get vlan_id
    element='vlan_id'
    vlan_id=`get_mgmtip`
    if [ -z $vlan_id ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty vlan id!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: vlan_id = $vlan_id" >>$NETCTRL_LOG 2>&1
    fi
}

# Reconfigure Mgmt IPs
function swup_reconfig_ips {
    # regenerate local mgmt interfaces.d file
    ${NETCTRL_DIR}/config_os.sh localip $vlan_id $ipaddr $mask $gateway
    # regenerate vlan mgmt interfaces.d file
    ${NETCTRL_DIR}/config_os.sh vlanip $vlan_id $vlan_ipaddr $vlan_mask $vlan_gateway
}

function swup_restore_ips {
    # set ip_addr
    element='ip_addr'
    value=$ipaddr
    set_mgmtip
    # get mask
    element='subnet'
    value=$mask
    set_mgmtip
    # get gateway
    element='gateway'
    value=$gateway
    set_mgmtip
    # get vlan_ip_addr
    element='vlan_ip_addr'
    value=$vlan_ipaddr
    set_mgmtip
    # get vlan_subnet
    element='vlan_subnet'
    value=$vlan_mask
    set_mgmtip
    # get vlan_gateway
    element='vlan_gateway'
    value=$vlan_gateway
    set_mgmtip
    # get vlan_id
    element='vlan_id'
    value=$vlan_id
    set_mgmtip

    swup_reconfig_ips
}

# Reload Mgmt IPs
function reload_mgmtip {
    # read mgmt ip from DB
    swup_backup_ips
    # reload mgmt ip
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Reloading Mgmt IP..." >>$NETCTRL_LOG 2>&1
    swup_reconfig_ips
}

#
##################################################
#
# Functions to handle actions.
#
##################################################
#
# install factory default
function factory_release {
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Removing any previous version..." >>$NETCTRL_LOG 2>&1
    swup_default
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Saving factory default image..." >>$NETCTRL_LOG 2>&1
    swup_image

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Installing default DBs..." >>$NETCTRL_LOG 2>&1
    swup_reset_default_db

    # make sure that public.old is removed
    sudo rm -rf /usr/local/optigo/www/public.old

    # update system DB
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Updating default system DB..." >>$NETCTRL_LOG 2>&1
    if [ -e ${NETCTRL_DIR}/version.txt ]; then
        CURR_VERSION=`cat ${NETCTRL_DIR}/version.txt`
        # set current_sw = CURR_VERSION
        value=$CURR_VERSION
        element='current_sw'
        set_version
        # set fd_version
        element='fd_version'
        set_version
    fi

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Installation Done!" >>$NETCTRL_LOG 2>&1

    # reset unit
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Reseting unit..." >>$NETCTRL_LOG 2>&1
    reset_unit
}

# install update
function update_release {
    # get sw1_version
    element='sw1_version'
    sw1_version=`get_version`
    if [ -z $sw1_version ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty version in 1st bank!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: sw1_version = $sw1_version" >>$NETCTRL_LOG 2>&1
    fi
    # get sw2_version
    element='sw2_version'
    sw2_version=`get_version`
    if [ -z $sw2_version ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty version in 2nd bank!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: sw2_version = $sw2_version" >>$NETCTRL_LOG 2>&1
    fi

    # backup mgmt ip
    swup_backup_ips
    sudo rm -rf /usr/local/optigo/www/public.*

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Checking new update..." >>$NETCTRL_LOG 2>&1
    swup_check
#    echo $HAS_UPDATE
    if [[ $HAS_UPDATE -ne 1 ]]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Nothing to update!" >>$NETCTRL_LOG 2>&1
        exit 1
    fi

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Upgrading ${NAME}..." >>$NETCTRL_LOG 2>&1
    swup_upgrade >>$NETCTRL_LOG 2>&1
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Upgrade Done!" >>$NETCTRL_LOG 2>&1

    # backup DBs to previous
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Backing DBs to previous ..." >>$NETCTRL_LOG 2>&1
    swup_backup_dbs

    # update system DB
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Updating system DB..." >>$NETCTRL_LOG 2>&1
    if [ -e ${NETCTRL_DIR}/version.txt ]; then
        CURR_VERSION=`cat ${NETCTRL_DIR}/version.txt`
        # set current_sw = CURR_VERSION
        value=$CURR_VERSION
        element='current_sw'
        set_version
        # set sw1_version = CURR_VERSION
        element='sw1_version'
        set_version
    fi

    if [ -e ${NETCTRL_PREV_DIR}/version.txt ]; then
        PREV_VERSION=`cat ${NETCTRL_PREV_DIR}/version.txt`
        # set previous_sw = PREV_VERSION
        element='previous_sw'
        value=$PREV_VERSION
        set_version

        if [ ! -z $sw1_version ]; then
            # set sw2_version
            element='sw2_version'
            value=$sw1_version
            set_version
        fi
    fi

    # migrate dbs
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Migrating DBs..." >>$NETCTRL_LOG 2>&1
    swup_migrate_db

    # restore mgmt ip
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Restoring Mgmt IP..." >>$NETCTRL_LOG 2>&1
    swup_restore_ips

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Update Done!" >>$NETCTRL_LOG 2>&1

#    has_usb=`mount | grep "/media/" | wc -l`
#    if [[ $has_usb -eq 0 ]]; then
        # reset unit
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Reseting unit..." >>$NETCTRL_LOG 2>&1
        reset_unit
#    else
        # finally umount the usb media safely
#        sudo sync
#        sudo umount -f `mount | grep "/media/" | awk '{print $1}'`
#    fi
}

# swap current with previous
function revert_release {
    # check if there is a prev release
    if [ ! -d ${NETCTRL_PREV_DIR} ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] ERROR: No previous version available! Aborting..." >>$NETCTRL_LOG 2>&1
        exit 1
    fi

    # Stop current running version of app
    stop_apps
    sudo rm -rf /usr/local/optigo/www/public.*

    # get sw1_version
    element='sw1_version'
    sw1_version=`get_version`
    if [ -z $sw1_version ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty version in 1st bank!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: sw1_version = $sw1_version" >>$NETCTRL_LOG 2>&1
    fi
    # get sw2_version
    element='sw2_version'
    sw2_version=`get_version`
    if [ -z $sw2_version ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] WARNING: Empty version in 2nd bank!" >>$NETCTRL_LOG 2>&1
    else
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: sw2_version = $sw2_version" >>$NETCTRL_LOG 2>&1
    fi

    # ONC-605: Added procedure to replace the older script with the new one in
    # order to prevent reverting the S8 FW.
    swup_s8_file=swup_ons_s8_fw.sh
    tmp_swup_s8=/tmp/$swup_s8_file
    actual_swup_s8=$TOP_DIR/$swup_s8_file
    swup_file=swup_update.sh
    tmp_swup=/tmp/$swup_file
    actual_swup=$TOP_DIR/$swup_file

    cp -p $actual_swup_s8 $tmp_swup_s8
    cp -p $actual_swup $tmp_swup

    # swap current release with previous release
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Reverting version..." >>$NETCTRL_LOG 2>&1
    swup_revert_releases

    # ONC-605: replace swup_ons_s8_fw.sh of the older version with the current.
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Updating HSW upgrade script" >>$NETCTRL_LOG 2>&1
    cp -p $tmp_swup_s8 $actual_swup_s8
    cp -p $tmp_swup $actual_swup

    cd ${NETCTRL_DIR}

    # ONC-605: comment out s8 update check for revert
    # ./swup_ons_s8_fw.sh update >>$NETCTRL_LOG 2>&1

    # run install script for prev version of app
    ./install.sh

    # swap current DBs with previous DBs
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Reverting DBs..." >>$NETCTRL_LOG 2>&1
    swup_revert_dbs

    # update system DB
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Updating system DB..." >>$NETCTRL_LOG 2>&1
    if [ -e ${NETCTRL_DIR}/version.txt ]; then
        CURR_VERSION=`cat ${NETCTRL_DIR}/version.txt`
        # set current_sw = CURR_VERSION
        element='current_sw'
        value=$CURR_VERSION
        set_version
    fi

    if [ -e ${NETCTRL_PREV_DIR}/version.txt ]; then
        PREV_VERSION=`cat ${NETCTRL_PREV_DIR}/version.txt`
        # set previous_sw = PREV_VERSION
        element='previous_sw'
        value=$PREV_VERSION
        set_version
    fi


    if [ ! -z $sw1_version ]; then
        # set sw1_version
        element='sw1_version'
        value=$sw1_version
        set_version
    fi

    if [ ! -z $sw2_version ]; then
        # set sw2_version
        element='sw2_version'
        value=$sw2_version
        set_version
    fi

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Revert Done!" >>$NETCTRL_LOG 2>&1
    # Start new version of app
#    start_apps

    # reset unit
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Reseting unit..." >>$NETCTRL_LOG 2>&1
    reset_unit
}

# swap_to_factory_default
function default_release {
    sudo rm -rf /usr/local/optigo/www/public.*
    stop_apps

    # get fd_version
    element='fd_version'
    fd_version=`get_version`
    if [ -z $fd_version ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] ERROR: Empty default version! Aborting..." >>$NETCTRL_LOG 2>&1
        exit 1
    fi
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: fd_version = $fd_version" >>$NETCTRL_LOG 2>&1
    if [[ -f ${NETCTRL_FW_DIR}/${PRODUCT}-${NAME}-${fd_version}.optg ]]; then
        filename=${PRODUCT}-${NAME}-${fd_version}.optg
    elif [[ -f ${NETCTRL_FW_DIR}/${PRODUCT}-${MODEL}-fw-${fd_version}.optg ]]; then
        filename=${PRODUCT}-${MODEL}-fw-${fd_version}.optg
    fi
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: fd_filename = $filename" >>$NETCTRL_LOG 2>&1

    # get factory default sw image
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Restoring default archive..." >>$NETCTRL_LOG 2>&1
    swup_restore
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Extracting archive..." >>$NETCTRL_LOG 2>&1
    swup_extract
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Installing ${NAME}..." >>$NETCTRL_LOG 2>&1
    swup_upgrade >>$NETCTRL_LOG 2>&1

    # install default dbs
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Installing default DBs..." >>$NETCTRL_LOG 2>&1
    swup_reset_default_db

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Updating default system DB..." >>$NETCTRL_LOG 2>&1
    value=$fd_version
    # set current_sw = fd_version
    element='current_sw'
    set_version
    # set fd_version
    element='fd_version'
    set_version

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Default installation Done!" >>$NETCTRL_LOG 2>&1
    # Start new version of app
#    start_apps

    # reset unit
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Reseting unit..." >>$NETCTRL_LOG 2>&1
    reset_unit
}

#
# reset config database to factory default version
#
function reset_sysdb {
    # set default version in system DB
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Updating default version in system DB..." >>$NETCTRL_LOG 2>&1
    if [ ! -e ${NETCTRL_DIR}/version.txt ]; then
        echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] ERROR: No version.txt file found. Aborting!" >>$NETCTRL_LOG 2>&1
        exit 1
    fi
    CURR_VERSION=`cat ${NETCTRL_DIR}/version.txt`

    # set current_sw = CURR_VERSION
    value=$CURR_VERSION
    element='current_sw'
    set_version
    # set fd_version
    element='fd_version'
    set_version

    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Reset version info Done!" >>$NETCTRL_LOG 2>&1
}

#
# check if there's a newer version of script
#
function check_self {
#    echo "Before: SWUP_NAME=$SWUP_NAME, SWUP_ARGS=$SWUP_ARGS"
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Saving new update image..." >>$NETCTRL_LOG 2>&1
    swup_backup
    echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] INFO: Extracting archive..." >>$NETCTRL_LOG 2>&1
    swup_extract
    # check if there is a new version of this script
    if [ -e "$SWUP_FILE" ] && [ "$SWUP_FILE" -nt "$TOP_DIR/swup_update.sh" ]; then
        # install a new version of this file or package
        echo "Found a new version of me, updating myself..."
        if [ -e "$SWUP_FILE_WRAPPER" ]; then
            mv -f "$SWUP_FILE_WRAPPER" "${SWUP_WRAPPER}"
            sudo chown ${NETCTRL_USER}:${NETCTRL_GROUP} "${SWUP_WRAPPER}"
            chmod 775 "${SWUP_WRAPPER}"
            exec "$SWUP_WRAPPER" $SWUP_ARGS
        else
            if [ -e "$SWUP_WRAPPER" ]; then
                exec "$SWUP_WRAPPER" $SWUP_ARGS 1
            else
                echo "`date '+%Y-%m-%d %H:%M:%S,%3N'` [NETC] ERROR: Update script not found! Aborting..." >>$NETCTRL_LOG 2>&1
                exit 1
            fi
        fi
        # now exit this old instance
        exit 0
    else
        echo "Script in version to update is older"
    fi

    echo "I'm VERSION $SWUP_VERSION, already the latest version."
}

#
# check if there's a new package archive to install
#
function update_packages {
    if [[ -f ${UPDATE_DIR}/packages.tar.bz2 ]]; then
        cd ${UPDATE_DIR}
        tar xjvf packages.tar.bz2 -C ${UPDATE_DIR}
        cd packages
        sudo ./install_packages.sh
        cd ${UPDATE_DIR}
    fi
}

# Input: element
# Output: value
function get_version {
    table=SYSTEM_CONFIG_TBL

    # get element
    local value=`system_db_get`
    echo $value
}

# Input: element value
function set_version {
    table=SYSTEM_CONFIG_TBL

    # set element
    system_db_set

    # get element
#    get_version
}

# Input: element
# Output: value
function get_mgmtip {
    table=IP_CONFIG_TBL

    # get element
    local value=`system_db_get`
    echo $value
}

# Input: element value
function set_mgmtip {
    table=IP_CONFIG_TBL

    # set element
    system_db_set

    # get element
#    get_mgmtip
}

function print_usage {
        echo "Usage: $0 {update|revert|default|reset|get|set|getip|setip} [filename|element|value]"
}

#
# main
#
# versioning info
SWUP_NAME="$0"
SWUP_ARGS="$@"
SWUP_VERSION=1.13
SWUP_FILE=${UPDATE_DIR}/swup_update.sh
SWUP_FILE_WRAPPER=${UPDATE_DIR}/swup_update_wrapper.sh
SWUP_WRAPPER=${NETCTRL_DIR}/swup_update_wrapper.sh

case $1 in
    factory)
        filename=
        factory_release
        ;;

    update)
        filename=$2
        if [ -z $3 ]; then
            # check if there's a new version of self
            check_self
        fi
        # need to killall apps before update starts...
        stop_apps
        # check if there're any new packages to add/install
        update_packages
        # update to new version of apps
        update_release
        ;;

    revert)
        filename=
        revert_release
        ;;

    default)
        filename=
        default_release
        ;;

    reset)
        reset_unit
        ;;

    sysdb)
        reset_sysdb
        ;;

    get)
        element=$2
        get_version
        ;;

    set)
        element=$2
        value=$3
        set_version
        ;;

    getip)
        element=$2
        get_mgmtip
        ;;

    setip)
        element=$2
        value=$3
        set_mgmtip
        ;;

    reloadip)
        reload_mgmtip
        ;;

    *)
        # For invalid arguments, print the usage message.
        print_usage
        exit 2
        ;;
esac

exit 0
